#    Copyright (C) 2003, 2004, Vampire Team <vampire-public@nongnu.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: RemoteTest.py,v 1.3 2004/01/11 18:57:41 nes Exp $

"""
Test a tarball on a host.

It mainly uses TarballUpload module (for sending tarballs) and RemoteAccess
module (for sending remote commands).
"""

import sys
import commands
import copy
import time

import common
from common import logger
import var_expansion
from RemoteAccess import RemoteAccess, RemoteAccessError
from TarballUpload import TarballUpload, TarballUploadError
from log.TestLogCommand import TestLogCommand
from log.TestLog import TestLog
from misc.thread_utilities import ThreadHandler

class RemoteTestError(Exception):
    """
    Exception used for RemoteTest module.
    """
    def __init__(self, str):
        self.__str = str
    def __str__(self):
        return self.__str

class RemoteTest(ThreadHandler):
    """
    Class inherited from thread. It tests a tarball on a remote host.
    """
    def __init__(self, scheduler, tarball, host):
        """
        Initialization of a test of a tarball on a remote host.

        It calls the expansion of the attributes of the tarball and the host,
        the salvage of the Md5sum of the local tarball.
        It sets the date of the beginning of the current test.
        @param logtarball: the log class (tarball, host, logs) used for tests.
        @param cond: the link between main thread and the RemoteTest
        instanciation.

        @type logtarball: TestLog
        @type cond: Condition

        @return: a RemoteTest instanciation.
        @rtype: RemoteTest
        """
        ThreadHandler.__init__(self, name = host.getName())
        self.__scheduler = scheduler
        self.__log = TestLog(tarball.getName(), host.getClassID(), host.getName())
        self.__tarball = tarball
        self.__host = host
        self.__expand_tarball()
        self.__expand_host()
        self.getTarball().md5 = self.getMd5()
        self.__log.setDate()
        self.__abort = False

    def getTarball(self):
        """
        Short accessor used to get the tarball

        @return: the tarball to test.
        @rtype: FinallTarball
        """
        return self.__tarball

    def getHost(self):
        """
        Short accessor used to get the host

        @return: the host used for the test.
        @rtype: FinalHost
        """
        return self.__host

    def addLog(self, log):
        """
        Short accessor used to add a log

        @parma log: Log to add
        @type log: TestLogCommand
        """
        self.__log.addLog(log)

    def expand(self, str):
        """
        Expand a string from the tarball and the host attributes.

        @param str: the variable to expand.
        @type str: string

        @return: the variable expanded.
        @rtype: string
        """
        return var_expansion.var_expand(self.getTarball(),
                                        self.getHost(), str)

    def __expand_host(self):
        """
        Expand all the attributes of the host.
        """
        for i in range(0, len(self.getHost().getConnectionCmd())):
            self.getHost().connection_cmd[i] = \
                self.expand(self.getHost().getConnectionCmd()[i])
        for i in range(0, len(self.getHost().getUploadCmd())):
            self.getHost().upload_cmd[i] = \
                self.getHost().getUploadCmd()[i][0], \
                self.expand(self.getHost().getUploadCmd()[i][1])
        self.getHost().tmpdir = self.expand(self.getHost().getTmpDir())
        self.getHost().md5local = self.expand(self.getHost().md5local)
        self.getHost().md5remote = self.expand(self.getHost().md5remote)

    def __expand_tarball(self):
        """
        Expand all the attributes of the tarball.
        """
        self.getTarball().name = self.expand(self.getTarball().getName())
        for i in self.getTarball().getBuildCmd():
            i.setName(self.expand(i.getName()))
            for j in i.getListCmdSuccess():
                j.setArg(self.expand(j.getArg()))
            for j in i.getListCmdFail():
                j.setArg(self.expand(j.getArg()))

    def getMd5(self):
        """
        Get the Md5sum of the tarball (local).

        @return: the Md5sum (or \"\" if it isn't possible).
        @rtype: string
        """
        if self.getHost().md5local == "":
            return ""
        (status, output) = commands.getstatusoutput(self.getHost().md5local)
        if status != 0:
            logger.info ("Error: can't get md5sum: '%s'" % output)
            return ""
        return output

    def getDeltaTime(self):
        """
        Calculate the period of time of the whole test.

        @return: the period of time between the date of beginning of the
        test and the current date.
        @rtype: string
        """
        delta = time.time() - time.mktime(time.strptime
                                          (self.__log.getDate(),
                                           "%a, %d %b %Y %H:%M:%S"))
        res = ""
        if delta // (60*60*24*7) != 0:
            res = "%d weeks, " % (delta // (60*60*24*7))
            delta = delta - ((delta // (60*60*24*7)) * 60*60*24*7)
        if res != "" or delta // (60*60*24) != 0:
            res += "%d days, " % (delta // (60*60*24))
            delta = delta - ((delta // (60*60*24)) * 60*60*24)
        if res != "" or delta // (60*60) != 0:
            res += "%dh " % (delta // (60*60))
            delta = delta - ((delta // (60*60)) * 60*60)
        res += "%dm " % (delta // (60))
        delta = delta - ((delta // (60)) * 60)
        res += "%.3fs" % delta
        return res

    def _end_run(self):
        """
        Free the current process, used before living RemoteTest.
        """
        self.__log.setTime(self.getDeltaTime())
        self.__scheduler.addEvent(("test_result", self.__log))
        self.__scheduler.addEvent(("terminated_task", self))

    def __get_infos(self, ListCmd, local=True):
        """
        Interpret information ('onsuccess' and 'onfail') commands.

        @param ListCmd: the list of infos commands.
        @param local: a boolean to know if it's a local or a distant
        information.

        @type ListCmd: CmdTask list
        @type local: boolean

        @return: the list of informations to keep.
        @rtype: string list
        """
        res = []
        for i in ListCmd:
            if i.getName() == "print":
                res.append(i.getArg())
            if i.getName() == "fetch":
                if local:
                    output = commands.getoutput("cat " + i.getArg())
                    res.append(i.getArg() + ":\n" +
                               unicode(output, sys.getfilesystemencoding(),
                                       'replace'))
                else:
                    try:
                        output = self.__access.send_command2("cat " +
                                                             i.getArg())
                        res.append(i.getArg() + ":\n" +
                                   unicode(output,
                                           sys.getfilesystemencoding(),
                                           'replace'))
                    except RemoteAccessError, e:
                        res.append("fetch: can't get %s." % i.getArg())
            if i.getName() == "abort":
                res.append("Test aborted.")
                self.__abort = True
        return res

    def __upload(self):
        """Upload the tarball calling TarballUpload module."""
        logger.info ("(%s) Uploading tarball `%s'." %
                     (self.getHost().getName(), self.getTarball().getName()))

        if common.options.do_nothing:
            self.addLog(TestLogCommand("upload", "special", 0,
                                   "Upload successful (dummy)"))
            return
            
        tu = TarballUpload(self.getHost(), self.getTarball(),
                           self.__access)
        try:
            good_command = tu.send()
        except TarballUploadError, e:
            logger.info ("Remote test: " + str(e))
            self.addLog(TestLogCommand("send", "error", -1, str(e)))
            self.__log.setFail()
            # If upload failed, abort the test
            self.__abort = True
            return
            self.addLog(TestLogCommand("upload", "special", 0,
                                   "Upload successful: %s" % good_command))

    def __local(self, buildcmd):
        """
        Execute a local command.

        @param buildcmd: the command to execute.
        @type buildcmd: TarballCommand
        """
        logger.info ("(localhost) Running `%s'." % buildcmd.getName())

        if common.options.do_nothing:
            self.addLog(TestLogCommand(buildcmd.getName(), buildcmd.getType()))
            return
        
        (status, output) = commands.getstatusoutput(buildcmd.getName())
        if status == 0:
            self.addLog(TestLogCommand
                        (buildcmd.getName(), buildcmd.getType(),
                         status, output,
                         self.__get_infos(buildcmd.getListCmdSuccess())))
        else:
            self.addLog(TestLogCommand
                        (buildcmd.getName(), buildcmd.getType(),
                         status, output,
                         self.__get_infos(buildcmd.getListCmdFail())))
            self.__log.setFail()

    def __remote(self, buildcmd):
        """
        Execute a remote command.

        @param buildcmd : the command to execute.
        @type buildcmd: TarballCommand
        """
        
        logger.info ("(%s) Running `%s'"
                     % (self.getHost().getName(), buildcmd.getName()))
        
        if common.options.do_nothing:
            self.addLog(TestLogCommand(buildcmd.getName(), buildcmd.getType()))
            return

        try:
            (output, status) = self.__access.send_command(buildcmd.getName())
        except RemoteAccessError, e:
            logger.info ("(%s) Failure on `%s' (%s)" %
                         (self.getHost().getName(), buildcmd.getName(), str(e)))
            self.addLog(TestLogCommand(buildcmd.getName(), "error", -1, str(e)))
            self.__log.setFail()
            # If command sending failed, abort the test
            self.__abort = True
            return
        if status == 0:
            self.addLog(TestLogCommand
                        (buildcmd.getName(), buildcmd.getType(),
                         status, output,
                         self.__get_infos(buildcmd.getListCmdSuccess(),
                                          False)))
        else:
            self.addLog(TestLogCommand
                        (buildcmd.getName(), buildcmd.getType(),
                         status, output,
                         self.__get_infos(buildcmd.getListCmdFail(), False)))
            self.__log.setFail()

    def run_(self):
        """
        Run the tarball testing on a given host.

            1. It connects to the remote host.
            2. It parses the list of commands to executes of the tarball
            (upload, local or remote commands).
            3. It closes the connection.
        """
        logger.info ("Starting remote test of '%s' on '%s'" %
                   (self.getTarball().getName(), self.getHost().getName()))

        # Connect to host
        if not logger.info ("Connect to '%s'" % self.getHost().getName()):
            try:
                self.__connect()
            except RemoteTestError, e:
                logger.info ("Remote test: " + str(e))
                self.addLog(TestLogCommand("connection", "error", -1, str(e)))
                self.__log.setFail()
                self._end_run()
                return

        # Execute build commands of the tarball
        for i in self.getTarball().getBuildCmd():
            if self.__abort:
                break
            if i.getType() == "special" and i.getName() == "upload":
                self.__upload()
            if i.getType() == "local":
                self.__local(i)
            if i.getType() == "remote":
                self.__remote(i)

        # Close the connection
        if not logger.info ("Close connection with '%s'"
                           % self.getHost().getName()):
            try:
                self.__access.close()
            except:
                logger.info ("Remote test: Vampire can't close the connection\
 with '%s' (already closed ?)." % self.getHost().getName())

        self._end_run()

    def __connect(self):
        """
        Connect to the remote host.

        It parses the list of connections methods of the host until it finds a
        good one.
        Then it goes in the temporary directory (create it if necessary).
        To finish, it exports the environment variables (CC, CXX, ...).
        """
        for i in self.getHost().getConnectionCmd():
            try:
                self.__access = RemoteAccess(i, 2)
                self.__access.connect()
                (output, status) = self.__access.send_command \
                                   ("test -d %s || mkdir %s && cd %s"
                                    % (self.getHost().getTmpDir(),
                                       self.getHost().getTmpDir(),
                                       self.getHost().getTmpDir()))
                for i in self.getHost().getEnv():
                    logger.debug ("(%s) Exporting %s = %s",
                                  self.getHost().getName(), i, self.getHost().getEnv()[i])
                    self.__access.send_command2("export %s=%s || setenv %s %s"
                                                % (i,
                                                   self.getHost().getEnv()[i],
                                                   i,
                                                   self.getHost().getEnv()[i]))
            except RemoteAccessError:
                pass
            else:
                if status == 0:
                    return
                raise RemoteTestError("%s: Can't access to temporary\
 directory: '%s'" % (self.getHost().getName(), self.getHost().getTmpDir()))
        raise RemoteTestError("All the connection commands have been tested,\
 but vampire can't connect to the remote host : '%s'"
                              % self.getHost().getName())

def main():
    """Testing purpose"""
    import os, threading, misc
    import parserXMLconf, parserXMLtarball, FinalHost, FinalTarball
    init_cmdline_options()
    hosts = parserXMLconf.ParserConf(os.path.expanduser( \
        common.CMDLINE_OPTIONS["host-file"]))
    tarballs = parserXMLtarball.ParserTarball(os.path.expanduser( \
        common.CMDLINE_OPTIONS["tarball-file"]))
    FinalHostList = FinalHost.transform(hosts.GetListClass(),
                                         hosts.GetListHost())
    FinalTarballList = FinalTarball.transform(tarballs.GetListClass(),
                                               tarballs.GetListTarball())

    r = RemoteTest(TestLog(FinalTarballList[0], FinalHostList[0]),
                    threading.Condition())
    r.start()

    while 1:
	print "[%d Threads: %s]" % (threading.activeCount(),
                                    threading.enumerate())
        time.sleep(2)

if __name__ == "__main__":
    main()
