#    Copyright (C) 2003, 2004, Vampire Team <vampire-public@nongnu.org>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: TarballUpload.py,v 1.1 2004/01/10 17:00:21 nes Exp $

"""
Send a tarball, trying all the sending methods needed.

It mainly uses functions of remote_upload module.
"""

import sys
import FinalHost
import FinalTarball
from RemoteAccess import RemoteAccess, RemoteAccessError
from remote_upload import *
from common import *
import var_expansion

class TarballUploadError(Exception):
    """
    Exeption used for TarballUpload module.
    """
    def __init__(self, str):
        self.__str = str
    def __str__(self):
        return self.__str

class TarballUpload:
    """
    Class used to send a tarball.
    """
    def __init__(self, host, tarball, access):
        """
        @param host: the distant host where tarball has to be sent.
        @param tarball: the tarball to send.
        @param access: the RemoteAccess instance to use.

        @type host: FinalHost
        @type tarball: FinalTarball
        @type access: RemoteAccess

        @return: a TarballUpload instanciation
        @rtype: TarballUpload
        """
        self.__host = host
        self.__tarball = tarball
        self.__access = access
        self.__log_error = ""

    ## Public
    def check(self):
        """
        Check if the tarball has been well received.

        It calls the checkmd5 function of remote_upload module.

        @return: If Md5 check is good returns True, else False.
        @rtype: boolean
        """
        if self.__tarball.getMd5() == "" or self.__host.md5remote == "":
            return True
        res = False
        try:
            res = checkmd5(self.__tarball.getMd5(),
                           self.__host.md5remote, self.__access)
        except RemoteAccessError, e:
            logger.info ("Error connecting: %s" % e)
        except RemoteUploadError, e:
            logger.info ("Error on md5 checking: %s" % e)

        return res

    def local(self, command):
        """
        Upload the tarball to the host (given in attributes).

        It uploads the tarball and checks the integrity of uploaded file.
        If it's badly uploaded, it tries again.

        @param command: the command used to upload the tarball (scp, ...).
        @type command: string

        @return: If upload is good, returns True, else False.
        @rtype: boolean
        """
        try:
            upload(command)
        except RemoteUploadError, e:
            logger.info ("(%s) Error uploading: %s" % (self.__host.getName(), e))
            self.__log_error += "Error uploading: %s \n" % e
            return False
        except RemoteAccessError, e:
            logger.info ("Connection lost with %s" % e)
            self.__log_error += "Connection lost with: %s \n" % e
            return False

        if self.check():
            logger.info ("md5sum OK.")
            return True
        else:
            logger.info ("md5sum FAILED.")
            return False

    def remote(self, command):
        """
        Download the tarball from the host (given in attributes).

        It downloads the tarball and checks the integrity of downloaded file.
        If it's badly downloaded, it tries again.

        @param command: the command used to download the tarball (scp, ...).
        @type command: string

        @return: If download is good, returns True, else False.
        @rtype: boolean
        """

        try:
            download(command, self.__access)
        except RemoteAccessError, e:
            logger.info ("Error in connection: %s" % e)
            self.__log_error += "Error in connection: %s\n" % e
            return False
        except RemoteUploadError, e:
            logger.info ("Error downloading from host: %s" % e)
            self.__log_error += "Error downloading from host: %s"  % e
            return False

        if self.check():
            logger.info ("md5sum OK.")
            return True
        else:
            logger.info ("md5sum FAILED.")
            return False            

        return False

    def send(self):
        """
        Send the tarball to the host (given in attributes).

        It parses the different upload commands (or remote download commands),
        and for each one, it tries to send the tarball. If it's not possible,
        it tries the next one.
        """
        logger.info ("Sending tarball '%s' to '%s' ..."
                   % (self.__tarball.getName(), self.__host.getName()))
        self.__log_error = ""
        for i in self.__host.getUploadCmd():
            if i == "":
                continue
            if i[0] == "local":
                if self.local(i[1]):
                    return "local: " + i[1]
            if i[0] == "remote":
                if self.remote(i[1]):
                    return "remote: " + i[1]
        logger.info ("(%s) Could not send `%s'."
                     % (self.__host.getName(), self.__tarball.getName()))
        raise TarballUploadError(self.__log_error)

def main():
    access = RemoteAccess("ssh ben@gouarec.dyndns.org bash")
    access.connect()
    host = FinalHost.FinalHost("server", {}, ("ssh sver", "ssh server"),
                               (("local", "scp README serv:~/"),
##                                ("remote", "rm -f README"),
                                ("local", "scp README gouarec.dyndns.org:~/"),
                                ("local",
                                 "scp README ben@gouarec.dyndns.org:~/")),
                               {}, "~/")
    tarball = FinalTarball.FinalTarball("README", "none", {}, [], "none",
                                        "8e0093d02317ec6fe61e5bd226988f70")
    tu = TarballUpload(host, tarball, access)
    try:
        tu.send()
    except TarballUploadError, e:
        print "TARBALL UPLOAD FAILED: %s" % e
    else:
        print "TARBALL UPLOAD OK"

if __name__ == "__main__":
    init_cmdline_options()
    main()
